﻿'GEOMETER:  A .NET class library written in Visual Basic Express 2012
'for analytic geometry, particularly vectors and complex numbers.

'Version 0.7.0 April 19 2013
'Copyright 2013 Wayne Matthew Syvinski

'TERMS OF USE

'*Disclaimer of Warranty

'    This software library (Geometer) comes with absolutely, positively NO WARRANTY, neither express or implied,  
'not even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. You use this software at your own risk.  
'In case of loss, no one owes you anything whatsoever. You have been warned. There is also NO GUARANTEE of 
'MATHEMATICAL CORRECTNESS. If you are going to use this software library in a production environment, 
'it is YOUR RESPONSIBILITY to verify that the methods, properties, and operators  return the correct values. 

'*Licensing

'    Geometer has a dual licensing scheme , and a patent provision.

'    If you use Geometer as-is, or make modifications to it other than making any class inheritable, 
'you may use it under the terms of the GNU Lesser General Public License version 3.

'    However, if you amend any class definition so as to make the class inheritable, you must use it under 
'the terms of the GNU General Public License version 3.

'    The GNU Lesser General Public License and the GNU General Public License are available at http://www.fsf.org

'    In addition, this software may not be incorporated into any software, whether as source or as linked library, 
'that is encumbered under any patent, unless all such patents have been released into the public domain.

'    When using Geometer under either aforementioned license, credit must be given to the copyright holder.

'    The copyright holder reserves the right to license Geometer under subsequent versions of the 
'GNU Lesser General Public License and the GNU General Public License.

'*Naming of Derivative Works

'    Both the GNU Lesser General Public License and the GNU General Public License allow developers to create derivative works.  
'However, if you create a derivative work, you must use a namespace other than geometer.  
'In addition, your derivative work may not be called "Geometer" or use the word geometer in its title.

Namespace geometer

    Public NotInheritable Class Vector

        Private _i As Double
        Private _j As Double
        Private _k As Double

        Public Sub New(i_component As Double, j_component As Double, k_component As Double)
            _i = i_component
            _j = j_component
            _k = k_component
        End Sub

        Public Sub New(i_component As Double, j_component As Double)
            _i = i_component
            _j = j_component
            _k = 0.0R
        End Sub

        Public Sub New()
            _i = 0.0R
            _j = 0.0R
            _k = 0.0R
        End Sub

        ''' <summary>
        ''' Use this constructor for vectors expressed in spherical coordinates
        ''' </summary>
        Public Sub New(r As Double, azimuth As Angle, polarAngle As Angle, Optional useInclination As Boolean = True)
            Dim workvector As Vector = New Point(r, azimuth, polarAngle, useInclination).ToVector
            _i = workvector.i
            _j = workvector.j
            _k = workvector.k
        End Sub

        ''' <summary>
        ''' Use this constructor for vectors expresed in cylindrical coordinates
        ''' </summary>    
        Public Sub New(rho As Double, azimuth As Angle, z As Double)
            Dim workvector As Vector = New Point(rho, azimuth, z).ToVector
            _i = workvector.i
            _j = workvector.j
            _k = workvector.k
        End Sub

        Public Sub New(orderedTriple As String)
            orderedTriple = Strings.Replace(orderedTriple, "(", "")
            orderedTriple = Strings.Replace(orderedTriple, ")", "")
            Dim components() As String = Strings.Split(orderedTriple, ",")
            If components.Length <> 3 Then
                Throw New ApplicationException("Exception in constructor for Class Vector.  You must pass a string that is parsable into three and only three numeric components.")
                Exit Sub
            End If

            Try
                _i = Convert.ToDouble(components(0))
                _j = Convert.ToDouble(components(1))
                _k = Convert.ToDouble(components(2))
            Catch ex As Exception
                Throw New ApplicationException("Input string could not be parsed into numeric components for a vector.")
            End Try

        End Sub

        Public Property i As Double
            Get
                Return _i
            End Get
            Set(value As Double)
                _i = i
            End Set
        End Property

        Public Property j As Double
            Get
                Return _j
            End Get
            Set(value As Double)
                _j = j
            End Set
        End Property

        Public Property k As Double
            Get
                Return _k
            End Get
            Set(value As Double)
                _k = k
            End Set
        End Property

        Public Shared Function Add(vectorarray() As Vector) As Vector

            Dim _isum As Double = 0.0R
            Dim _jsum As Double = 0.0R
            Dim _ksum As Double = 0.0R

            For Each vec As Vector In vectorarray
                _isum += vec.i
                _jsum += vec.j
                _ksum += vec.k
            Next

            Return New Vector(_isum, _jsum, _ksum)

        End Function

        Public Shared Function AngleBetweenVectors(vec1 As Vector, vec2 As Vector) As Angle

            Return New Angle(Math.Asin(vec1.Cross(vec2).Magnitude / (vec1.Magnitude * vec2.Magnitude)))

        End Function

        Public Function Dot(vec As Vector) As Double

            Return (Me.i * vec.i) + (Me.j * vec.j) + (Me.k * vec.k)

        End Function

        Public Function Cross(vec As Vector) As Vector

            Dim ux As Double = Me.i
            Dim uy As Double = Me.j
            Dim uz As Double = Me.k

            Dim vx As Double = vec.i
            Dim vy As Double = vec.j
            Dim vz As Double = vec.k

            Return New Vector((uy * vz) - (uz * vy), (uz * vx) - (ux * vz), (ux * vy) - (uy * vx))

        End Function

        Public Function ScalarTripleProduct(a As Vector, b As Vector, c As Vector) As Double
            Return a.Dot((b.Cross(c)))
        End Function

        Public Function VectorTripleProduct(a As Vector, b As Vector, c As Vector) As Vector
            Return a.Cross(b.Cross(c))
        End Function

        Public Shared ReadOnly Property ui As Vector
            Get
                Return New Vector(1, 0, 0)
            End Get
        End Property

        Public Shared ReadOnly Property uj As Vector
            Get
                Return New Vector(0, 1, 0)
            End Get
        End Property

        Public Shared ReadOnly Property uk As Vector
            Get
                Return New Vector(0, 0, 1)
            End Get
        End Property

        Public ReadOnly Property Magnitude As Double
            Get
                Return Math.Sqrt((Me.i ^ 2.0R) + (Me.j ^ 2.0R) + (Me.k ^ 2.0R))
            End Get
        End Property

        Public ReadOnly Property UnitVector As Vector
            Get
                Dim mag As Double = Me.Magnitude
                Return New Vector(Me.i / mag, Me.j / mag, Me.k / mag)
            End Get
        End Property

        Public ReadOnly Property ijk As String
            Get
                Return Me.i.ToString & "i" & If(Me.j >= 0.0R, " + ", " - ") & Math.Abs(Me.j).ToString & "j" & If(Me.k >= 0.0R, " + ", " - ") & Math.Abs(Me.k).ToString & "k"
            End Get
        End Property

        Public ReadOnly Property OrdTriple As String
            Get
                Return "(" & Me.i.ToString & "," & Me.j.ToString & "," & Me.k.ToString & ")"
            End Get
        End Property

        Public ReadOnly Property ToPoint As Point
            Get
                Return New Point(Me.i, Me.j, Me.k)
            End Get
        End Property

        Public Shared Operator +(vec1 As Vector, vec2 As Vector) As Vector
            Return New Vector(vec1.i + vec2.i, vec1.j + vec2.j, vec1.k + vec2.k)
        End Operator

        Public Shared Operator -(vec1 As Vector, vec2 As Vector) As Vector
            Return New Vector(vec1.i - vec2.i, vec1.j - vec2.j, vec1.k - vec2.k)
        End Operator

        Public Shared Operator -(vec As Vector) As Vector
            Return New Vector(-vec.i, -vec.j, -vec.k)
        End Operator

        Public Shared Operator *(scalar As Double, vec As Vector) As Vector
            Return New Vector(scalar * vec.i, scalar * vec.j, scalar * vec.k)
        End Operator

        Public Shared Operator *(vec As Vector, scalar As Double) As Vector
            Return New Vector(scalar * vec.i, scalar * vec.j, scalar * vec.k)
        End Operator

        Public Shared Operator /(vec As Vector, scalar As Double) As Vector
            Return New Vector(vec.i / scalar, vec.j / scalar, vec.k / scalar)
        End Operator

        Public Shared Operator =(vec1 As Vector, vec2 As Vector) As Boolean
            If (vec1.i = vec2.i) And (vec1.j = vec2.j) And (vec1.k = vec2.k) Then
                Return True
            Else
                Return False
            End If
        End Operator

        Public Shared Operator <>(vec1 As Vector, vec2 As Vector) As Boolean
            If (vec1.i = vec2.i) And (vec1.j = vec2.j) And (vec1.k = vec2.k) Then
                Return False
            Else
                Return True
            End If
        End Operator



    End Class

End Namespace
